@@ -25,9 +25,11 @@
 #' @param strict_version_check (Optional) Setting this to FALSE is unsupported and should only be done when advised by technical support.
 #' @param proxy (Optional) A \code{character} string specifying the proxy path.
 #' @param https (Optional) Set this to TRUE to use https instead of http.
+#' @param cacert (Optional) Path to a CA bundle file with root and intermediate certificates of trusted CAs.
 #' @param insecure (Optional) Set this to TRUE to disable SSL certificate checking.
 #' @param username (Optional) Username to login with.
 #' @param password (Optional) Password to login with.
+#' @param use_spnego (Optional) Set this to TRUE to enable SPNEGO authentication.
 #' @param cookies (Optional) Vector(or list) of cookies to add to request.
 #' @param context_path (Optional) The last part of connection URL: http://<ip>:<port>/<context_path>
 #' @param ignore_config (Optional) A \code{logical} value indicating whether a search for a .h2oconfig file should be conducted or not. Default value is FALSE.
@@ -63,7 +65,8 @@ h2o.init <- function(ip = "localhost", port = 54321, name = NA_character_, start
                      max_mem_size = NULL, min_mem_size = NULL,
                      ice_root = tempdir(), log_dir = NA_character_, log_level = NA_character_,
                      strict_version_check = TRUE, proxy = NA_character_,
-                     https = FALSE, insecure = FALSE, username = NA_character_, password = NA_character_,
+                     https = FALSE, cacert = NA_character_, insecure = FALSE,
+                     username = NA_character_, password = NA_character_, use_spnego = FALSE,
                      cookies = NA_character_, context_path = NA_character_, ignore_config = FALSE,
                      extra_classpath = NULL, jvm_custom_args = NULL,
                      bind_to_localhost = TRUE) {
@@ -89,6 +92,9 @@ h2o.init <- function(ip = "localhost", port = 54321, name = NA_character_, start
         if(insecure == FALSE && "init.verify_ssl_certificates" %in% colnames(h2oconfig)){
           insecure <- as.logical(trimws(toupper(as.character(h2oconfig$init.verify_ssl_certificates))))
         }
+        if(is.na(cacert) && "init.cacert" %in% colnames(h2oconfig)){
+          cacert <- trimws(as.character(h2oconfig$cacert))
+        }
         if(is.na(cookies) && "init.cookies" %in% colnames(h2oconfig)){
           cookies <- as.vector(trimws(strsplit(as.character(h2oconfig$init.cookies),";")[[1]]))
         }
@@ -98,6 +104,9 @@ h2o.init <- function(ip = "localhost", port = 54321, name = NA_character_, start
         if(is.na(password) && "init.password" %in% colnames(h2oconfig)){
           password <- trimws(as.character(h2oconfig$init.password))
         }
+        if(use_spnego == FALSE && "init.use_spnego" %in% colnames(h2oconfig)){
+          use_spnego <- as.logical(trimws(toupper(as.character(h2oconfig$init.use_spnego))))
+        }
       }
     }
 
@@ -139,10 +148,14 @@ h2o.init <- function(ip = "localhost", port = 54321, name = NA_character_, start
     stop("`proxy` must be a character string or NA_character_")
   if(!is.logical(https) || length(https) != 1L || is.na(https))
     stop("`https` must be TRUE or FALSE")
+  if(!is.character(cacert) || !nzchar(cacert))
+    stop("`cacert` must be a character string or NA_character_")
   if(!is.logical(insecure) || length(insecure) != 1L || is.na(insecure))
     stop("`insecure` must be TRUE or FALSE")
-  if(https != insecure)
-    stop("`https` and `insecure` must both be TRUE to enable HTTPS")
+  if(!is.logical(use_spnego) || length(use_spnego) != 1L || is.na(use_spnego))
+    stop("`use_spnego` must be TRUE or FALSE")
+  else if (!is.na(use_spnego) && use_spnego && !is.na(username))
+    stop("Specify either `use_spnego` or `username` and `password`")
   if(!is.character(username) || !nzchar(username))
     stop("`username` must be a character string or NA_character_")
   if (!is.character(password) || !nzchar(password))
@@ -169,8 +182,10 @@ h2o.init <- function(ip = "localhost", port = 54321, name = NA_character_, start
   if (nchar(doc_port)) port <- as.numeric(doc_port)
 
   warnNthreads <- FALSE
-  tmpConn <- new("H2OConnection", ip = ip, port = port, name = name, proxy = proxy, https = https, insecure = insecure,
-    username = username, password = password,cookies = cookies, context_path = context_path)
+  tmpConn <- new("H2OConnection", ip = ip, port = port, name = name, proxy = proxy,
+                 https = https, cacert = cacert, insecure = insecure,
+                 username = username, password = password, use_spnego = use_spnego,
+                 cookies = cookies, context_path = context_path)
   if (!h2o.clusterIsUp(tmpConn)) {
     if (!startH2O)
       stop("Cannot connect to H2O server. Please check that H2O is running at ", h2o.getBaseURL(tmpConn))
@@ -218,8 +233,10 @@ h2o.init <- function(ip = "localhost", port = 54321, name = NA_character_, start
       stop("Can only start H2O launcher if IP address is localhost.")
   }
 
-  conn <- new("H2OConnection", ip = ip, port = port, name = .h2o.jar.env$name, proxy = proxy, https = https, insecure = insecure,
-    username = username, password = password,cookies = cookies, context_path = context_path)
+  conn <- new("H2OConnection", ip = ip, port = port, name = .h2o.jar.env$name, proxy = proxy,
+              https = https, cacert = cacert, insecure = insecure,
+              username = username, password = password, use_spnego = use_spnego,
+              cookies = cookies, context_path = context_path)
   assign("SERVER", conn, .pkg.env)
   cat(" Connection successful!\n\n")
   .h2o.jar.env$port <- port #Ensure right port is called when quitting R
@@ -271,9 +288,11 @@ h2o.init <- function(ip = "localhost", port = 54321, name = NA_character_, start
 #' @param strict_version_check (Optional) Setting this to FALSE is unsupported and should only be done when advised by technical support.
 #' @param proxy (Optional) A \code{character} string specifying the proxy path.
 #' @param https (Optional) Set this to TRUE to use https instead of http.
+#' @param cacert Path to a CA bundle file with root and intermediate certificates of trusted CAs.
 #' @param insecure (Optional) Set this to TRUE to disable SSL certificate checking.
 #' @param username (Optional) Username to login with.
 #' @param password (Optional) Password to login with.
+#' @param use_spnego (Optional) Set this to TRUE to enable SPNEGO authentication.
 #' @param cookies (Optional) Vector(or list) of cookies to add to request.
 #' @param context_path (Optional) The last part of connection URL: http://<ip>:<port>/<context_path>
 #' @param config (Optional) A \code{list} describing connection parameters. Using \code{config} makes \code{h2o.connect} ignore
@@ -293,9 +312,9 @@ h2o.init <- function(ip = "localhost", port = 54321, name = NA_character_, start
 #' #h2o.connect(config = c(strict_version_check = FALSE, config))
 #' }
 #' @export
-h2o.connect <- function(ip = "localhost", port = 54321, strict_version_check = TRUE,
-                        proxy = NA_character_ , https = FALSE, insecure = FALSE,
-                        username = NA_character_, password = NA_character_,
+h2o.connect <- function(ip = "localhost", port = 54321, strict_version_check = TRUE, proxy = NA_character_,
+                        https = FALSE, cacert = NA_character_, insecure = FALSE,
+                        username = NA_character_, password = NA_character_, use_spnego = FALSE,
                         cookies = NA_character_,
                         context_path = NA_character_,
                         config = NULL) {
@@ -309,8 +328,8 @@ h2o.connect <- function(ip = "localhost", port = 54321, strict_version_check = T
  } else {
    # Pass arguments directly
    h2o.init(ip=ip, port=port, strict_version_check=strict_version_check,
-            proxy=proxy, https=https, insecure=insecure, password=password,
-            username=username, cookies=cookies, context_path=context_path)
+            proxy=proxy, https=https, cacert=cacert, insecure=insecure, password=password,
+            username=username, use_spnego=use_spnego, cookies=cookies, context_path=context_path)
  }
 }
 
@@ -567,20 +586,20 @@ h2o.clusterStatus <- function() {
       error = function(err) {
         print(err)
         stop("You have a 32-bit version of Java. H2O works best with 64-bit Java.\n",
-        "Please download the latest Java SE JDK 8 from the following URL:\n",
-        "http://www.oracle.com/technetwork/java/javase/downloads/jdk8-downloads-2133151.html")
+        "Please download the latest Java SE JDK from the following URL:\n",
+        "https://www.oracle.com/technetwork/java/javase/downloads/index.html")
       }
     )
   jver_error <- .h2o.check_java_version(jver);
   if (!is.null(jver_error)) {
     stop(jver_error, "\n",
-    "Please download the latest Java SE JDK 8 from the following URL:\n",
-    "http://www.oracle.com/technetwork/java/javase/downloads/jdk8-downloads-2133151.html")
+    "Please download the latest Java SE JDK from the following URL:\n",
+    "https://www.oracle.com/technetwork/java/javase/downloads/index.html")
   }
   if(any(grepl("Client VM", jver))) {
     warning("You have a 32-bit version of Java. H2O works best with 64-bit Java.\n",
-            "Please download the latest Java SE JDK 8 from the following URL:\n",
-            "http://www.oracle.com/technetwork/java/javase/downloads/jdk8-downloads-2133151.html")
+            "Please download the latest Java SE JDK from the following URL:\n",
+            "https://www.oracle.com/technetwork/java/javase/downloads/index.html")
 
     # Set default max_memory to be 1g for 32-bit JVM.
     if(is.null(max_memory)) max_memory = "1g"
