@@ -157,7 +157,7 @@ h2o.getModel <- function(model_id) {
   model_category <- json$output$model_category
   if (is.null(model_category))
     model_category <- "Unknown"
-  else if (!(model_category %in% c("Unknown", "Binomial", "Multinomial", "Ordinal", "Regression", "Clustering", "AutoEncoder", "DimReduction", "WordEmbedding", "CoxPH", "AnomalyDetection")))
+  else if (!(model_category %in% c("Unknown", "Binomial", "Multinomial", "Ordinal", "Regression", "Clustering", "AutoEncoder", "DimReduction", "WordEmbedding", "CoxPH", "AnomalyDetection", "TargetEncoder")))
     stop(paste0("model_category, \"", model_category,"\", missing in the output"))
   Class <- paste0("H2O", model_category, "Model")
   model <- json$output[!(names(json$output) %in% c("__meta", "model_category"))]
@@ -169,6 +169,10 @@ h2o.getModel <- function(model_id) {
   if (model_category %in% c("Binomial", "Multinomial", "Ordinal", "Regression")) { # add the missing metrics manually where
     model$coefficients <- model$coefficients_table[,2]
     names(model$coefficients) <- model$coefficients_table[,1]
+    if (!is.null(model$random_coefficients_table)) {
+      model$random_coefficients <- model$random_coefficients_table[,2]
+      names(model$random_coefficients) <- model$random_coefficients_table[,1]
+    }
   }
   parameters <- list()
   allparams  <- list()
@@ -219,11 +223,20 @@ h2o.getModel <- function(model_id) {
     parameters$x <- setdiff(parameters$x, parameters$y)
     allparams$x <- setdiff(allparams$x, allparams$y)
   }
-
   allparams$ignored_columns <- NULL
   allparams$response_column <- NULL
   parameters$ignored_columns <- NULL
   parameters$response_column <- NULL
+  if (identical("glm", json$algo) && allparams$HGLM) {
+    .newH2OModel(Class         = Class,
+                 model_id      = model_id,
+                 algorithm     = json$algo,
+                 parameters    = parameters,
+                 allparameters = allparams,
+                 have_pojo     = FALSE,
+                 have_mojo     = FALSE,
+                 model         = model)
+  } else {
   .newH2OModel(Class         = Class,
                model_id      = model_id,
                algorithm     = json$algo,
@@ -232,6 +245,7 @@ h2o.getModel <- function(model_id) {
                have_pojo     = json$have_pojo,
                have_mojo     = json$have_mojo,
                model         = model)
+  }
 }
 
 #'
@@ -392,3 +406,44 @@ h2o.download_mojo <- function(model, path=getwd(), get_genmodel_jar=FALSE, genmo
   }
   return(paste0(model_id,".zip"))
 }
+
+#'
+#' Download the model in binary format.
+#'
+#' @param model An H2OModel
+#' @param path The path where binary file should be downloaded. Downloaded to current directory by default.
+#'
+#' @examples
+#' \dontrun{
+#' library(h2o)
+#' h <- h2o.init()
+#' fr <- as.h2o(iris)
+#' my_model <- h2o.gbm(x = 1:4, y = 5, training_frame = fr)
+#' h2o.download_model(my_model)  # save to the current working directory
+#' }
+#' @export
+h2o.download_model <- function(model, path=NULL) {
+
+    if(!is.null(path) && !(is.character(path))){
+      stop("The 'path' variable should be of type character")
+    }
+    if(!is.null(path) && !(file.exists(path))){
+      stop(paste0("'path',",path,", to save pojo cannot be found."))
+    }
+    if(is.null(path)){
+      path = getwd()
+    }
+    
+    #Get model id
+    model_id <- model@model_id
+    
+    #prepare suffix to get the right endpoint
+    urlSuffix = paste0(.h2o.__MODELS, ".fetch.bin/", model_id)
+    modelname = gsub("[+\\-* !@#$%^&()={}\\[\\]|;:'\"<>,.?/]","_",model_id,perl=T)
+    
+    #Path to save model, if `path` is provided
+    file_path <- file.path(path, paste0(modelname))
+    writeBin(.h2o.doSafeGET(urlSuffix = urlSuffix, binary = TRUE), file_path, useBytes = TRUE)
+    
+    return(paste0(file_path))
+}
